@@ -1,6 +1,6 @@
 package Text::CSV_XS;
 
-# Copyright (c) 2007-2009 H.Merijn Brand.  All rights reserved.
+# Copyright (c) 2007-2010 H.Merijn Brand.  All rights reserved.
 # Copyright (c) 1998-2001 Jochen Wiedmann. All rights reserved.
 # Portions Copyright (c) 1997 Alan Citterman. All rights reserved.
 #
@@ -30,7 +30,7 @@ use DynaLoader ();
 use Carp;
 
 use vars   qw( $VERSION @ISA );
-$VERSION = "0.59";
+$VERSION = "0.74";
 @ISA     = qw( DynaLoader );
 bootstrap Text::CSV_XS $VERSION;
 
@@ -59,13 +59,17 @@ my %def_attr = (
     sep_char		=> ',',
     eol			=> '',
     always_quote	=> 0,
+    quote_space		=> 1,
+    quote_null		=> 1,
     binary		=> 0,
     keep_meta_info	=> 0,
     allow_loose_quotes	=> 0,
     allow_loose_escapes	=> 0,
     allow_whitespace	=> 0,
     blank_is_undef	=> 0,
+    empty_is_undef	=> 0,
     verbatim		=> 0,
+    auto_diag		=> 0,
     types		=> undef,
 
     _EOF		=> 0,
@@ -76,34 +80,61 @@ my %def_attr = (
     _ERROR_INPUT	=> undef,
     _COLUMN_NAMES	=> undef,
     _BOUND_COLUMNS	=> undef,
+    _AHEAD		=> undef,
     );
 my $last_new_err = Text::CSV_XS->SetDiag (0);
 
+sub _check_sanity
+{
+    my $attr = shift;
+    for (qw( sep_char quote_char escape_char )) {
+	exists $attr->{$_} && defined $attr->{$_} && $attr->{$_} =~ m/[\r\n]/ and
+	    return 1003;
+	}
+    $attr->{allow_whitespace} and
+      (defined $attr->{quote_char}  && $attr->{quote_char}  =~ m/^[ \t]$/) ||
+      (defined $attr->{escape_char} && $attr->{escape_char} =~ m/^[ \t]$/) and
+	return 1002;
+    return 0;
+    } # _check_sanity
+
 sub new
 {
-    $last_new_err =
-	"usage: my \$csv = Text::CSV_XS->new ([{ option => value, ... }]);";
+    $last_new_err = SetDiag (undef, 1000,
+	"usage: my \$csv = Text::CSV_XS->new ([{ option => value, ... }]);");
+
     my $proto = shift;
+    my $class = ref ($proto) || $proto	or  return;
+    @_ > 0 &&   ref $_[0] ne "HASH"	and return;
     my $attr  = shift || {};
-    my $class = ref ($proto) || $proto	or return;
+
     for (keys %{$attr}) {
 	if (m/^[a-z]/ && exists $def_attr{$_}) {
-	    $] >= 5.008002 && m/_char$/ and utf8::decode $attr->{$_};
+	    $] >= 5.008002 && m/_char$/ and utf8::decode ($attr->{$_});
 	    next;
 	    }
 #	croak?
-	$last_new_err = "Unknown attribute '$_'";
+	$last_new_err = SetDiag (undef, 1000, "INI - Unknown attribute '$_'");
+	$attr->{auto_diag} and error_diag ();
 	return;
 	}
-    $last_new_err = SetDiag (undef, 0);
+
     my $self  = {%def_attr, %{$attr}};
+    if (my $ec = _check_sanity ($self)) {
+	$last_new_err = SetDiag (undef, $ec);
+	$attr->{auto_diag} and error_diag ();
+	return;
+	}
+
+    $last_new_err = SetDiag (undef, 0);
     defined $\ && !exists $attr->{eol} and $self->{eol} = $\;
     bless $self, $class;
     defined $self->{types} and $self->types ($self->{types});
     $self;
     } # new
 
-my %_cache_id = (	# Keep in sync with XS!
+# Keep in sync with XS!
+my %_cache_id = ( # Only expose what is accessed from within PM
     quote_char		=>  0,
     escape_char		=>  1,
     sep_char		=>  2,
@@ -115,36 +146,42 @@ my %_cache_id = (	# Keep in sync with XS!
     allow_double_quoted	=>  8,
     allow_whitespace	=>  9,
     blank_is_undef	=> 10,
-
     eol			=> 11,	# 11 .. 18
-    eol_len		=> 19,
-    eol_is_cr		=> 20,
-    has_types		=> 21,
     verbatim		=> 22,
-
-    _is_bound		=> 23,	# 23 .. 26
+    empty_is_undef	=> 23,
+    auto_diag		=> 24,
+    quote_space		=> 25,
+    quote_null		=> 31,
+    _is_bound		=> 26,	# 26 .. 29
     );
 
+# A `character'
 sub _set_attr_C
+{
+    my ($self, $name, $val, $ec) = @_;
+    defined $val or $val = 0;
+    $] >= 5.008002 and utf8::decode ($val);
+    $self->{$name} = $val;
+    $ec = _check_sanity ($self) and
+	croak ($self->SetDiag ($ec));
+    $self->_cache_set ($_cache_id{$name}, $val);
+    } # _set_attr_C
+
+# A flag
+sub _set_attr_X
 {
     my ($self, $name, $val) = @_;
     defined $val or $val = 0;
-    $] >= 5.008002 and utf8::decode $val;
     $self->{$name} = $val;
-    $self->{_CACHE} or return;
-    my @cache = unpack "C*", $self->{_CACHE};
-    $cache[$_cache_id{$name}] = unpack "C", $val;
-    $self->{_CACHE} = pack "C*", @cache;
-    } # _set_attr_C
+    $self->_cache_set ($_cache_id{$name}, 0 + $val);
+    } # _set_attr_X
 
+# A number
 sub _set_attr_N
 {
     my ($self, $name, $val) = @_;
     $self->{$name} = $val;
-    my @cache = unpack "C*", $self->{_CACHE};
-    my $i = $_cache_id{$name};
-    $cache[$i++] = $_ for unpack "C*", pack "N", $val;
-    $self->{_CACHE} = pack "C*", @cache;
+    $self->_cache_set ($_cache_id{$name}, 0 + $val);
     } # _set_attr_N
 
 # Accessor methods.
@@ -152,14 +189,20 @@ sub _set_attr_N
 sub quote_char
 {
     my $self = shift;
-    @_ and $self->_set_attr_C ("quote_char", shift);
+    if (@_) {
+	my $qc = shift;
+	$self->_set_attr_C ("quote_char", $qc);
+	}
     $self->{quote_char};
     } # quote_char
 
 sub escape_char
 {
     my $self = shift;
-    @_ and $self->_set_attr_C ("escape_char", shift);
+    if (@_) {
+	my $ec = shift;
+	$self->_set_attr_C ("escape_char", $ec);
+	}
     $self->{escape_char};
     } # escape_char
 
@@ -176,19 +219,8 @@ sub eol
     if (@_) {
 	my $eol = shift;
 	defined $eol or $eol = "";
-	my $eol_len = length $eol;
 	$self->{eol} = $eol;
-	$self->{_CACHE} or return;
-	my @cache = unpack "C*", $self->{_CACHE};
-	if (($cache[$_cache_id{eol_len}] = $eol_len) < 8) {
-	    $cache[$_cache_id{eol_is_cr}] = $eol eq "\r" ? 1 : 0;
-	    }
-	else {
-	    $cache[$_cache_id{eol_is_cr}] = 0;
-	    }
-	$eol .= "\0\0\0\0\0\0\0\0";
-	$cache[$_cache_id{eol} + $_] = unpack "C", substr $eol, $_, 1 for 0 .. 7;
-	$self->{_CACHE} = pack "C*", @cache;
+	$self->_cache_set ($_cache_id{eol}, $eol);
 	}
     $self->{eol};
     } # eol
@@ -196,59 +228,94 @@ sub eol
 sub always_quote
 {
     my $self = shift;
-    @_ and $self->_set_attr_C ("always_quote", shift);
+    @_ and $self->_set_attr_X ("always_quote", shift);
     $self->{always_quote};
     } # always_quote
 
+sub quote_space
+{
+    my $self = shift;
+    @_ and $self->_set_attr_X ("quote_space", shift);
+    $self->{quote_space};
+    } # quote_space
+
+sub quote_null
+{
+    my $self = shift;
+    @_ and $self->_set_attr_X ("quote_null", shift);
+    $self->{quote_null};
+    } # quote_null
+
 sub binary
 {
     my $self = shift;
-    @_ and $self->_set_attr_C ("binary", shift);
+    @_ and $self->_set_attr_X ("binary", shift);
     $self->{binary};
     } # binary
 
 sub keep_meta_info
 {
     my $self = shift;
-    @_ and $self->_set_attr_C ("keep_meta_info", shift);
+    @_ and $self->_set_attr_X ("keep_meta_info", shift);
     $self->{keep_meta_info};
     } # keep_meta_info
 
 sub allow_loose_quotes
 {
     my $self = shift;
-    @_ and $self->_set_attr_C ("allow_loose_quotes", shift);
+    @_ and $self->_set_attr_X ("allow_loose_quotes", shift);
     $self->{allow_loose_quotes};
     } # allow_loose_quotes
 
 sub allow_loose_escapes
 {
     my $self = shift;
-    @_ and $self->_set_attr_C ("allow_loose_escapes", shift);
+    @_ and $self->_set_attr_X ("allow_loose_escapes", shift);
     $self->{allow_loose_escapes};
     } # allow_loose_escapes
 
 sub allow_whitespace
 {
     my $self = shift;
-    @_ and $self->_set_attr_C ("allow_whitespace", shift);
+    if (@_) {
+	my $aw = shift;
+	$aw and
+	  (defined $self->{quote_char}  && $self->{quote_char}  =~ m/^[ \t]$/) ||
+	  (defined $self->{escape_char} && $self->{escape_char} =~ m/^[ \t]$/) and
+	    croak ($self->SetDiag (1002));
+	$self->_set_attr_X ("allow_whitespace", $aw);
+	}
     $self->{allow_whitespace};
     } # allow_whitespace
 
 sub blank_is_undef
 {
     my $self = shift;
-    @_ and $self->_set_attr_C ("blank_is_undef", shift);
+    @_ and $self->_set_attr_X ("blank_is_undef", shift);
     $self->{blank_is_undef};
     } # blank_is_undef
 
+sub empty_is_undef
+{
+    my $self = shift;
+    @_ and $self->_set_attr_X ("empty_is_undef", shift);
+    $self->{empty_is_undef};
+    } # empty_is_undef
+
 sub verbatim
 {
     my $self = shift;
-    @_ and $self->_set_attr_C ("verbatim", shift);
+    @_ and $self->_set_attr_X ("verbatim", shift);
     $self->{verbatim};
     } # verbatim
 
+sub auto_diag
+{
+    my $self = shift;
+    @_ and $self->_set_attr_X ("auto_diag", shift);
+    $self->{auto_diag};
+    } # auto_diag
+
 # status
 #
 #   object method returning the success or failure of the most recent
@@ -279,24 +346,46 @@ sub error_input
 
 # erro_diag
 #
-#   If (and only if) an error occured, this function returns a code that
+#   If (and only if) an error occurred, this function returns a code that
 #   indicates the reason of failure
 
 sub error_diag
 {
     my $self = shift;
-    my @diag = (0, $last_new_err, 0);
+    my @diag = (0 + $last_new_err, $last_new_err, 0);
 
-    unless ($self && ref $self) {	# Class method or direct call
-	$last_new_err and $diag[0] = 1000;
-	}
-    elsif ($self->isa (__PACKAGE__) && exists $self->{_ERROR_DIAG}) {
+    if ($self && ref $self && # Not a class method or direct call
+	 $self->isa (__PACKAGE__) && exists $self->{_ERROR_DIAG}) {
 	@diag = (0 + $self->{_ERROR_DIAG}, $self->{_ERROR_DIAG});
 	exists $self->{_ERROR_POS} and $diag[2] = 1 + $self->{_ERROR_POS};
 	}
+
     my $context = wantarray;
-    unless (defined $context) {	# Void context
-	$diag[0] and print STDERR "# CSV_XS ERROR: $diag[0] - $diag[1]\n";
+    unless (defined $context) {	# Void context, auto-diag
+	if ($diag[0] && $diag[0] != 2012) {
+	    my $msg = "# CSV_XS ERROR: $diag[0] - $diag[1] \@ pos $diag[2]\n";
+	    if ($self && ref $self) {	# auto_diag
+
+		my $lvl = $self->{auto_diag};
+		if ($lvl < 2) {
+		    my @c = caller (2);
+		    if (@c >= 11 && $c[10] && ref $c[10] eq "HASH") {
+			my $hints = $c[10];
+			(exists $hints->{autodie} && $hints->{autodie} or
+			 exists $hints->{"guard Fatal"} &&
+			!exists $hints->{"no Fatal"}) and
+			    $lvl++;
+			# Future releases of autodie will probably set $^H{autodie}
+			#  to "autodie @args", like "autodie :all" or "autodie open"
+			#  so we can/should check for "open" or "new"
+			}
+		    }
+		$lvl > 1 ? die $msg : warn $msg;
+		}
+	    else {	# called without args in void context
+		warn $msg;
+		}
+	    }
 	return;
 	}
     return $context ? @diag : $diag[1];
@@ -493,31 +582,21 @@ Text::CSV_XS - comma-separated values manipulation routines
 
  use Text::CSV_XS;
 
- $csv = Text::CSV_XS->new ();          # create a new object
- $csv = Text::CSV_XS->new (\%attr);    # create a new object
+ my @rows;
+ my $csv = Text::CSV_XS->new ({ binary => 1 }) or
+     die "Cannot use CSV: ".Text::CSV->error_diag ();
+ open my $fh, "<:encoding(utf8)", "test.csv" or die "test.csv: $!";
+ while (my $row = $csv->getline ($fh)) {
+     $row->[2] =~ m/pattern/ or next; # 3rd field should match
+     push @rows, $row;
+     }
+ $csv->eof or $csv->error_diag ();
+ close $fh;
 
- $status  = $csv->combine (@columns);  # combine columns into a string
- $line    = $csv->string ();           # get the combined string
-
- $status  = $csv->parse ($line);       # parse a CSV string into fields
- @columns = $csv->fields ();           # get the parsed fields
-
- $status       = $csv->status ();      # get the most recent status
- $bad_argument = $csv->error_input (); # get the most recent bad argument
- $diag         = $csv->error_diag ();  # if an error occured, explains WHY
-
- $status = $csv->print ($io, $colref); # Write an array of fields
-                                       # immediately to a file $io
- $colref = $csv->getline ($io);        # Read a line from file $io,
-                                       # parse it and return an array
-                                       # ref of fields
- $csv->bind_columns (@refs);           # Set return fields for getline ()
- $csv->column_names (@names);          # Set column names for getline_hr ()
- $ref = $csv->getline_hr ($io);        # getline (), but returns a hashref
- $eof = $csv->eof ();                  # Indicate if last parse or
-                                       # getline () hit End Of File
-
- $csv->types (\@t_array);              # Set column types
+ $csv->eol ("\r\n");
+ open $fh, ">:encoding(utf8)", "new.csv" or die "new.csv: $!";
+ $csv->print ($fh, $_) for @rows;
+ close $fh or die "new.csv: $!";
 
 =head1 DESCRIPTION
 
@@ -531,7 +610,7 @@ perhaps better called ASV (anything separated values) rather than just CSV.
 
 =head2 Embedded newlines
 
-B<Important Note>: The default behavior is to only accept ascii characters.
+B<Important Note>: The default behavior is to only accept ASCII characters.
 This means that fields can not contain newlines. If your data contains
 newlines embedded in fields, or characters above 0x7e (tilde), or binary data,
 you *must* set C<< binary => 1 >> in the call to C<new ()>.  To cover the widest
@@ -546,7 +625,7 @@ usage:
      $csv->parse ($_);
      my @fields = $csv->fields ();
 
-will break, as the while might read broken lines, as that doesn't care
+will break, as the while might read broken lines, as that does not care
 about the quoting. If you need to support embedded newlines, the way to go
 is either
 
@@ -564,11 +643,8 @@ or, more safely in perl 5.6 and up
 =head2 Unicode (UTF8)
 
 On parsing (both for C<getline ()> and C<parse ()>), if the source is
-marked being UTF8, then parsing that source will mark all fields that
-are marked binary will also be marked UTF8.
-
-On combining (C<print ()> and C<combine ()>), if any of the combining
-fields was marked UTF8, the resulting string will be marked UTF8.
+marked being UTF8, then all fields that are marked binary will also be
+be marked UTF8.
 
 For complete control over encoding, please use Text::CSV::Encoded:
 
@@ -586,6 +662,16 @@ For complete control over encoding, please use Text::CSV::Encoded:
     # combine () and print () accept UTF8 marked data
     # parse () and getline () return UTF8 marked data
 
+On combining (C<print ()> and C<combine ()>), if any of the combining
+fields was marked UTF8, the resulting string will be marked UTF8. Note
+however that all fields C<before> the first field that was marked UTF8
+and contained 8-bit characters that were not upgraded to UTF8, these
+will be bytes in the resulting string too, causing errors. If you pass
+data of different encoding, or you don't know if there is different
+encoding, force it to be upgraded before you pass them on:
+
+    $csv->print ($fh, [ map { utf8::upgrade (my $x = $_); $x } @data ]);
+
 =head1 SPECIFICATION
 
 While no formal specification for CSV exists, RFC 4180 1) describes a common
@@ -655,7 +741,7 @@ QUOTATION MARK) (to give some examples of what might look promising) are
 therefor not allowed.
 
 If you use perl-5.8.2 or higher, these three attributes are utf8-decoded, to
-increase the likelyhood of success. This way U+00FE will be allowed as a
+increase the likelihood of success. This way U+00FE will be allowed as a
 quote character.
 
 =item *
@@ -712,14 +798,18 @@ See also CAVEATS
 =item allow_whitespace
 
 When this option is set to true, whitespace (TAB's and SPACE's)
-surrounding the separation character is removed when parsing. So
-lines like:
+surrounding the separation character is removed when parsing. If
+either TAB or SPACE is one of the three major characters C<sep_char>,
+C<quote_char>, or C<escape_char> it will not be considered whitespace.
+
+So lines like:
 
   1 , "foo" , bar , 3 , zapp
 
 are now correctly parsed, even though it violates the CSV specs.
+
 Note that B<all> whitespace is stripped from start and end of each
-field. That would make is more a I<feature> than a way to be able
+field. That would make it more a I<feature> than a way to be able
 to parse bad CSV lines, as
 
  1,   2.0,  3,   ape  , monkey
@@ -750,6 +840,20 @@ be parsed as
 
  ("1", "", undef, " ", "2")
 
+=item empty_is_undef
+
+Going one step further than C<blank_is_undef>, this attribute converts
+all empty fields to undef, so
+
+ 1,"",," ",2
+
+is read as
+
+ (1, undef, undef, " ", 2)
+
+Note that this only effects fields that are I<really> empty, not fields
+that are empty after stripping allowed whitespace. YMMV.
+
 =item quote_char
 
 The char used for quoting fields containing blanks, by default the
@@ -836,9 +940,23 @@ of the I<types> method below.
 
 By default the generated fields are quoted only, if they need to, for
 example, if they contain the separator. If you set this attribute to
-a TRUE value, then all fields will be quoted. This is typically easier
-to handle in external applications. (Poor creatures who aren't using
-Text::CSV_XS. :-)
+a TRUE value, then all defined fields will be quoted. This is typically
+easier to handle in external applications. (Poor creatures who are not
+using Text::CSV_XS. :-)
+
+=item quote_space
+
+By default, a space in a field would trigger quotation. As no rule
+exists this to be forced in CSV, nor any for the opposite, the default
+is true for safety. You can exclude the space from this trigger by
+setting this attribute to 0.
+
+=item quote_null
+
+By default, a NULL byte in a field would be escaped. This attribute
+enables you to treat the NULL byte as a simple binary character in
+binary mode (the C<{ binary => 1 }> is set). The default is true.
+You can prevent NULL escapes by setting this attribute to 0.
 
 =item keep_meta_info
 
@@ -870,7 +988,7 @@ Imagine a file format like
 where, the line ending is a very specific "#\r\n", and the sep_char
 is a ^ (caret). None of the fields is quoted, but embedded binary
 data is likely to be present. With the specific line ending, that
-shouldn't be too hard to detect.
+should not be too hard to detect.
 
 By default, Text::CSV_XS' parse function however is instructed to only
 know about "\n" and "\r" to be legal line endings, and so has to deal
@@ -882,6 +1000,20 @@ as if \n is just nothing more than a binary character.
 For parse () this means that the parser has no idea about line ending
 anymore, and getline () chomps line endings on reading.
 
+=item auto_diag
+
+Set to true will cause C<error_diag ()> to be automatically be called
+in void context upon errors.
+
+In case of error C<2012 - EOF>, this call will be void.
+
+If set to a value greater than 1, it will die on errors instead of
+warn.
+
+Future extensions to this feature will include more reliable auto-detection
+of the C<autodie> module being enabled, which will raise the value of
+C<auto_diag> with C<1> on the moment the error is detected.
+
 =back
 
 To sum it up,
@@ -896,13 +1028,17 @@ is equivalent to
      sep_char            => ',',
      eol                 => $\,
      always_quote        => 0,
+     quote_space         => 1,
+     quote_null	         => 1,
      binary              => 0,
      keep_meta_info      => 0,
      allow_loose_quotes  => 0,
      allow_loose_escapes => 0,
      allow_whitespace    => 0,
      blank_is_undef      => 0,
+     empty_is_undef      => 0,
      verbatim            => 0,
+     auto_diag           => 0,
      });
 
 For all of the above mentioned flags, there is an accessor method
@@ -920,11 +1056,34 @@ If the C<new ()> constructor call fails, it returns C<undef>, and makes
 the fail reason available through the C<error_diag ()> method.
 
  $csv = Text::CSV_XS->new ({ ecs_char => 1 }) or
-     die Text::CSV_XS->error_diag ();
+     die "".Text::CSV_XS->error_diag ();
 
 C<error_diag ()> will return a string like
 
- "Unknown attribute 'ecs_char'"
+ "INI - Unknown attribute 'ecs_char'"
+
+=head2 print
+
+ $status = $csv->print ($io, $colref);
+
+Similar to C<combine () + string () + print>, but more efficient. It
+expects an array ref as input (not an array!) and the resulting string is
+not really created, but immediately written to the I<$io> object, typically
+an IO handle or any other object that offers a I<print> method. Note, this
+implies that the following is wrong in perl 5.005_xx and older:
+
+ open FILE, ">", "whatever";
+ $status = $csv->print (\*FILE, $colref);
+
+as in perl 5.005 and older, the glob C<\*FILE> is not an object, thus it
+does not have a print method. The solution is to use an IO::File object or
+to hide the glob behind an IO::Wrap object. See L<IO::File> and L<IO::Wrap>
+for details.
+
+For performance reasons the print method does not create a result string.
+In particular the I<$csv-E<gt>string ()>, I<$csv-E<gt>status ()>,
+I<$csv->fields ()> and I<$csv-E<gt>error_input ()> methods are meaningless
+after executing this method.
 
 =head2 combine
 
@@ -937,28 +1096,6 @@ retrieve the resultant CSV string.  Upon failure, the value returned by
 C<string ()> is undefined and C<error_input ()> can be called to retrieve an
 invalid argument.
 
-=head2 print
-
- $status = $csv->print ($io, $colref);
-
-Similar to combine, but it expects an array ref as input (not an array!)
-and the resulting string is not really created, but immediately written
-to the I<$io> object, typically an IO handle or any other object that
-offers a I<print> method. Note, this implies that the following is wrong:
-
- open FILE, ">", "whatever";
- $status = $csv->print (\*FILE, $colref);
-
-The glob C<\*FILE> is not an object, thus it doesn't have a print
-method. The solution is to use an IO::File object or to hide the
-glob behind an IO::Wrap object. See L<IO::File(3)> and L<IO::Wrap(3)>
-for details.
-
-For performance reasons the print method doesn't create a result string.
-In particular the I<$csv-E<gt>string ()>, I<$csv-E<gt>status ()>,
-I<$csv->fields ()> and I<$csv-E<gt>error_input ()> methods are meaningless
-after executing this method.
-
 =head2 string
 
  $line = $csv->string ();
@@ -966,20 +1103,6 @@ after executing this method.
 This object function returns the input to C<parse ()> or the resultant CSV
 string of C<combine ()>, whichever was called more recently.
 
-=head2 parse
-
- $status = $csv->parse ($line);
-
-This object function decomposes a CSV string into fields, returning
-success or failure.  Failure can result from a lack of argument or the
-given CSV string is improperly formatted.  Upon success, C<fields ()> can
-be called to retrieve the decomposed fields .  Upon failure, the value
-returned by C<fields ()> is undefined and C<error_input ()> can be called
-to retrieve the invalid argument.
-
-You may use the I<types ()> method for setting column types. See the
-description below.
-
 =head2 getline
 
  $colref = $csv->getline ($io);
@@ -995,6 +1118,20 @@ reference to an empty list.
 The I<$csv-E<gt>string ()>, I<$csv-E<gt>fields ()> and I<$csv-E<gt>status ()>
 methods are meaningless, again.
 
+=head2 parse
+
+ $status = $csv->parse ($line);
+
+This object function decomposes a CSV string into fields, returning
+success or failure.  Failure can result from a lack of argument or the
+given CSV string is improperly formatted.  Upon success, C<fields ()> can
+be called to retrieve the decomposed fields .  Upon failure, the value
+returned by C<fields ()> is undefined and C<error_input ()> can be called
+to retrieve the invalid argument.
+
+You may use the I<types ()> method for setting column types. See the
+description below.
+
 =head2 getline_hr
 
 The C<getline_hr ()> and C<column_names ()> methods work together to allow
@@ -1025,7 +1162,7 @@ string C<"\cAUNDEF\cA">, so
   $hr = $csv->getline_hr ($io);
 
 Will set C<$hr->{"\cAUNDEF\cA"}> to the 1st field, C<$hr->{""}> to the
-2nd field, and C<$hr->{name}> to the 4th field, discarding the 2rd field.
+2nd field, and C<$hr->{name}> to the 4th field, discarding the 3rd field.
 
 C<column_names ()> croaks on invalid arguments.
 
@@ -1095,7 +1232,7 @@ Set field type to string.
  @columns = $csv->fields ();
 
 This object function returns the input to C<combine ()> or the resultant
-decomposed fields of C successfull <parse ()>, whichever was called more
+decomposed fields of C successful <parse ()>, whichever was called more
 recently.
 
 Note that the return value is undefined after using C<getline ()>, which
@@ -1111,7 +1248,7 @@ was called more recently.
 
 For each field, a meta_info field will hold flags that tell something about
 the field returned by the C<fields ()> method or passed to the C<combine ()>
-method. The flags are bitwise-or'd like:
+method. The flags are bit-wise-or'd like:
 
 =over 4
 
@@ -1171,7 +1308,7 @@ C<combine ()> or C<parse ()>, whichever was called more recently.
  $error_str    = "" . $csv->error_diag ();
  ($cde, $str, $pos) = $csv->error_diag ();
 
-If (and only if) an error occured, this function returns the diagnostics
+If (and only if) an error occurred, this function returns the diagnostics
 of that error.
 
 If called in void context, it will print the internal error code and the
@@ -1187,8 +1324,8 @@ If called in scalar context, it will return the diagnostics in a single
 scalar, a-la $!. It will contain the error code in numeric context, and
 the diagnostics message in string context.
 
-When called as a class method or a direct function call, the error diag
-is that of the last C<new ()> call.
+When called as a class method or a direct function call, the error
+diagnostics is that of the last C<new ()> call.
 
 =head2 SetDiag
 
@@ -1213,7 +1350,17 @@ as the API may change in future releases.
 
 =head1 EXAMPLES
 
-An example for parsing CSV strings:
+Reading a CSV file line by line:
+
+  my $csv = Text::CSV_XS->new ({ binary => 1 });
+  open my $fh, "<", "file.csv" or die "file.csv: $!";
+  while (my $row = $csv->getline ($fh)) {
+      # do something with @$row
+      }
+  $csv->eof or $csv->error_diag;
+  close $fh or die "file.csv: $!";
+
+Parsing CSV strings:
 
   my $csv = Text::CSV_XS->new ({ keep_meta_info => 1, binary => 1 });
 
@@ -1227,12 +1374,25 @@ An example for parsing CSV strings:
           }
       }
   else {
-      my $err = $csv->error_input;
-      print STDERR "parse () failed on argument: ", $err, "\n";
+      print STDERR "parse () failed on argument: ",
+	  $csv->error_input, "\n";
       $csv->error_diag ();
       }
 
-An example for creating CSV files:
+An example for creating CSV files using the C<print ()> method, like in
+dumping the content of a database ($dbh) table ($tbl) to CSV:
+
+  my $csv = Text::CSV_XS->new ({ binary => 1, eol => $/ });
+  open my $fh, ">", "$tbl.csv" or die "$tbl.csv: $!";
+  my $sth = $dbh->prepare ("select * from $tbl");
+  $sth->execute;
+  $csv->print ($fh, $sth->{NAME_lc});
+  while (my $row = $sth->fetch) {
+      $csv->print ($fh, $row) or $csv->error_diag;
+      }
+  close $fh or die "$tbl.csv: $!";
+
+or using the slower C<combine ()> and C<string ()> methods:
 
   my $csv = Text::CSV_XS->new;
 
@@ -1242,49 +1402,58 @@ An example for creating CSV files:
       'You said, "Hello!"',   5.67,
       '"Surely"',   '',   '3.14159');
   if ($csv->combine (@sample_input_fields)) {
-      my $string = $csv->string;
-      print $csv_fh "$string\n";
+      print $csv_fh $csv->string, "\n";
       }
   else {
-      my $err = $csv->error_input;
-      print "combine () failed on argument: ", $err, "\n";
+      print "combine () failed on argument: ",
+	  $csv->error_input, "\n";
       }
   close $csv_fh or die "hello.csv: $!";
 
-Or using the C<print ()> method, which is faster like in
-dumping the content of a database ($dbh) table ($tbl) to CSV:
+For more extended examples, see the C<examples/> sub-directory in the
+original distribution or the git repository at
+http://repo.or.cz/w/Text-CSV_XS.git?a=tree;f=examples. The following files
+can be found there:
 
-  my $csv = Text::CSV_XS->new ({ binary => 1, eol => $/ });
-  open my $fh, ">", "$tbl.csv" or die "$tbl.csv: $!";
-  my $sth = $dbh->prepare ("select * from $tbl");
-  $sth->execute;
-  $csv->print ($fh, $sth->{NAME_lc});
-  while (my $row = $sth->fetch) {
-      $csv->print ($fh, $row) or ...;
-      }
-  close $fh or die "$tbl.csv: $!";
+=over 2
 
-Reading a CSV file line by line:
+=item parser-xs.pl
 
-  my $csv = Text::CSV_XS->new ({ binary => 1 });
-  open my $fh, "<", "file.csv" or die "file.csv: $!";
-  while (my $row = $csv->getline ($fh)) {
-      # do something with @$row
-      }
-  $csv->eof or $csv->error_diag;
-  close $fh or die "file.csv: $!";
+This can be used as a boilerplate to `fix' bad CSV and parse beyond errors.
 
-For more extended examples, see the C<examples/> subdirectory in the
-original distribution. Included is C<examples/parser-xs.pl>, that could
-be used to `fix' bad CSV and parse beyond errors.
+  $ perl examples/parser-xs.pl bad.csv >good.csv
 
-  perl examples/parser-xs.pl bad.csv >good.csv
+=item csv-check
+
+This is a command-line tool that uses parser-xs.pl techniques to check the
+CSV file and report on its content.
+
+  $ csv-check files/utf8.csv
+  Checked with examples/csv-check 1.2 using Text::CSV_XS 0.61
+  OK: rows: 1, columns: 2
+      sep = <,>, quo = <">, bin = <1>
+
+=item csv2xls
+
+A script to convert CSV to Microsoft Excel. This requires L<Date::Calc>
+and L<Spreadsheet::WriteExcel>. The converter accepts various options and
+can produce UTF-8 Excel files.
+
+=item csvdiff
+
+A script that provides colorized diff on sorted CSV files, assuming first
+line is header and first field is the key. Output options include colorized
+ANSI escape codes or HTML.
+
+  $ csvdiff --html --output=diff.html file1.csv file2.csv
+
+=back
 
 =head1 CAVEATS
 
 C<Text::CSV_XS> is not designed to detect the characters used for field
 separation and quoting. The parsing is done using predefined settings. In
-the examples subdirectory, you can find scripts that demonstrate how you
+the examples sub-directory, you can find scripts that demonstrate how you
 can try to detect these characters yourself.
 
 =head2 Microsoft Excel
@@ -1301,13 +1470,6 @@ setting in it, so checking the locale is no solution.
 
 =over 2
 
-=item More tests
-
-For all possible errors, there should be a test.
-
-All XS code should be covered in the test cases, except for perl
-internal failure, like failing to store a hash value.
-
 =item More Errors & Warnings
 
 New extensions ought to be clear and concise in reporting what error
@@ -1332,17 +1494,12 @@ combine ()/string () combination.
 Requests for adding means (methods) that combine C<combine ()> and
 C<string ()> in a single call will B<not> be honored. Likewise for
 C<parse ()> and C<fields ()>. Given the trouble with embedded newlines,
-Using C<getline ()> and C<print ()> instead is the prefered way to go.
-
-=item Unicode
-
-We probably need many more tests to check if all edge-cases are covered.
-See t/50_utf8.t.
+Using C<getline ()> and C<print ()> instead is the preferred way to go.
 
 =item Parse the whole file at once
 
 Implement a new methods that enables the parsing of a complete file
-at once, returning a lis of hashes. Possible extension to this could
+at once, returning a list of hashes. Possible extension to this could
 be to enable a column selection on the call:
 
    my @AoH = $csv->parse_file ($filename, { cols => [ 1, 4..8, 12 ]});
@@ -1365,6 +1522,11 @@ The hard-coding of characters and character ranges makes this module
 unusable on EBCDIC system. Using some #ifdef structure could enable
 these again without loosing speed. Testing would be the hard part.
 
+Opening EBCDIC encode files on ASCII+ systems is likely to succeed
+using Encode's cp37, cp1047, or posix-bc:
+
+  open my $fh, "<:encoding(cp1047)", "ebcdic_file.csv" or die "...";
+
 =back
 
 =head1 Release plan
@@ -1390,14 +1552,22 @@ No guarantees, but this is what I have in mind right now:
 
 Still under construction ...
 
-If an error occured, C<$csv->error_diag ()> can be used to get more information
+If an error occurred, C<$csv->error_diag ()> can be used to get more information
 on the cause of the failure. Note that for speed reasons, the internal value
 is never cleared on success, so using the value returned by C<error_diag ()> in
-normal cases - when no error occured - may cause unexpected results.
+normal cases - when no error occurred - may cause unexpected results.
 
-Currently errors as described below are available. I've tried to make the error
-itself explanatory enough, but more descriptions will be added. For most of
-these errors, the first three capitals describe the error category:
+If the constructor failed, the cause can be found using C<error_diag ()> as a
+class method, like C<Text::CSV_XS->error_diag ()>.
+
+C<$csv->error_diag ()> is automatically called upon error when the contractor
+was called with C<auto_diag> set to 1 or 2, or when C<autodie> is in effect.
+When set to 1, this will cause a C<warn ()> with the error message, when set
+to 2, it will C<die ()>. C<2012 - EOF> is excluded from C<auto_diag> reports.
+
+Currently errors as described below are available. I have tried to make the
+error itself explanatory enough, but more descriptions will be added. For most
+of these errors, the first three capitals describe the error category:
 
 =over 2
 
@@ -1438,6 +1608,16 @@ HashRef parse related error.
 The separation character cannot be equal to either the quotation character
 or the escape character, as that will invalidate all parsing rules.
 
+=item 1002 "INI - allow_whitespace with escape_char or quote_char SP or TAB"
+
+Using C<allow_whitespace> when either C<escape_char> or C<quote_char> is
+equal to SPACE or TAB is too ambiguous to allow.
+
+=item 1003 "INI - \r or \n in main attr not allowed"
+
+Using default C<eol> characters in either C<sep_char>, C<quote_char>, or
+C<escape_char> is not allowed.
+
 =item 2010 "ECR - QUO char inside quotes followed by CR not part of EOL"
 
 When C<eol> has been set to something specific, other than the default,
@@ -1467,7 +1647,7 @@ has been selected with the constructor.
 Sequences like C<1,"foo\rbar",2> are only allowed when the binary option
 has been selected with the constructor.
 
-=item 2023 "EIQ - QUO character not allowed
+=item 2023 "EIQ - QUO character not allowed"
 
 Sequences like C<"foo "bar" baz",quux> and C<2023,",2008-04-05,"Foo, Bar",\n>
 will cause this error.
@@ -1523,17 +1703,17 @@ exhausted before the quote is found, that field is not terminated.
 
 =item 3006 "EHR - bind_columns () did not pass enough refs for parsed fields"
 
-=item 3007 "EHR - bind_columns needs refs to writeable scalars"
+=item 3007 "EHR - bind_columns needs refs to writable scalars"
 
-=item 3008 "EHR - unexpected error in bound fields
+=item 3008 "EHR - unexpected error in bound fields"
 
 =back
 
 =head1 SEE ALSO
 
-L<perl(1)>, L<IO::File(3)>, L<IO::Handle(3)>, L<IO::Wrap(3)>,
-L<Text::CSV(3)>, L<Text::CSV_PP(3)>, L<Text::CSV::Encoded>,
-L<Text::CSV::Separator(3)>, and L<Spreadsheet::Read(3)>.
+L<perl>, L<IO::File>, L<IO::Handle>, L<IO::Wrap>, L<Text::CSV>,
+L<Text::CSV_PP>, L<Text::CSV::Encoded>, L<Text::CSV::Separator>,
+and L<Spreadsheet::Read>.
 
 =head1 AUTHORS and MAINTAINERS
 
@@ -1550,12 +1730,12 @@ and the print and getline methods. See ChangeLog releases 0.10 through
 
 H.Merijn Brand F<E<lt>h.m.brand@xs4all.nlE<gt>> cleaned up the code,
 added the field flags methods, wrote the major part of the test suite,
-completed the documentation, fixed some RT bugs. See ChangeLog releases
-0.25 and on.
+completed the documentation, fixed some RT bugs and added all the allow
+flags. See ChangeLog releases 0.25 and on.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2007-2009 H.Merijn Brand for PROCURA B.V.
+Copyright (C) 2007-2010 H.Merijn Brand for PROCURA B.V.
 Copyright (C) 1998-2001 Jochen Wiedmann. All rights reserved.
 Portions Copyright (C) 1997 Alan Citterman. All rights reserved.
 
